@@ -89,11 +89,6 @@ class ValidationError(DKIMException):
     """Validation error."""
     pass
 
-def _remove(s, t):
-    i = s.find(t)
-    assert i >= 0
-    return s[:i] + s[i+len(t):]
-
 def select_headers(headers, include_headers):
     """Select message header fields to be signed/verified.
 
@@ -119,14 +114,17 @@ def select_headers(headers, include_headers):
         lastindex[h] = i
     return sign_headers
 
+FWS = r'(?:\r\n\s+)?'
+RE_BTAG = re.compile(r'([; ]b'+FWS+r'=)(?:'+FWS+r'[a-zA-Z0-9+/=])*(?:\r\n\Z)?')
+
 def hash_headers(hasher, canonicalize_headers, headers, include_headers,
-                 sigheaders, sig):
+                 sigheader, sig):
     """Update hash for signed message header fields."""
     sign_headers = select_headers(headers,include_headers)
     # The call to _remove() assumes that the signature b= only appears
     # once in the signature header
     cheaders = canonicalize_headers.canonicalize_headers(
-        [(sigheaders[0][0], _remove(sigheaders[0][1], sig[b'b']))])
+        [(sigheader[0], RE_BTAG.sub(b'\\1',sigheader[1]))])
     # the dkim sig is hashed with no trailing crlf, even if the
     # canonicalization algorithm would add one.
     for x,y in sign_headers + [(x, y.rstrip()) for x,y in cheaders]:
@@ -161,7 +159,7 @@ def validate_signature_fields(sig):
         not sig[b'i'].endswith(sig[b'd']) or
         sig[b'i'][-len(sig[b'd'])-1] not in ('@', '.', 64, 46)):
         raise ValidationError(
-            "i= domain is not a subdomain of d= (i=%s d=%d)" %
+            "i= domain is not a subdomain of d= (i=%s d=%s)" %
             (sig[b'i'], sig[b'd']))
     if b'l' in sig and re.match(br"\d{,76}$", sig['l']) is None:
         raise ValidationError(
@@ -304,8 +302,6 @@ class DKIM(object):
     self.should_not_sign = set(DKIM.SHOULD_NOT)
     #: Header fields to sign an extra time to prevent additions.
     self.frozen_sign = set(DKIM.FROZEN)
-    #: Signature parameters of last sign or verify
-    self.signature_fields = {}
 
   def add_frozen(self,s):
     """ Add headers not in should_not_sign to frozen_sign.
@@ -329,10 +325,14 @@ class DKIM(object):
       self.headers, self.body = rfc822_parse(message)
     else:
       self.headers, self.body = [],''
-    #: The DKIM signing domain last signed or verified
+    #: The DKIM signing domain last signed or verified.
     self.domain = None
-    #: The DKIM key selector last signed or verified
+    #: The DKIM key selector last signed or verified.
     self.selector = 'default'
+    #: Signature parameters of last sign or verify.  To parse
+    #: a DKIM-Signature header field that you have in hand,
+    #: use L{dkim.util.parse_tag_value}.
+    self.signature_fields = {}
     #: The list of headers last signed or verified.  Each header
     #: is a name,value tuple.  FIXME: The headers are canonicalized.
     #: This could be more useful as original headers.
@@ -390,7 +390,7 @@ class DKIM(object):
   #: @raise DKIMException: when the message, include_headers, or key are badly
   #: formed.
   def sign(self, selector, domain, privkey, identity=None,
-        canonicalize=(b'simple',b'simple'), include_headers=None, length=False):
+        canonicalize=(b'relaxed',b'simple'), include_headers=None, length=False):
     try:
         pk = parse_pem_private_key(privkey)
     except UnparsableKeyError as e:
@@ -435,25 +435,33 @@ class DKIM(object):
         (b't', str(int(time.time())).encode('ascii')),
         (b'h', b" : ".join(include_headers)),
         (b'bh', bodyhash),
-        (b'b', b""),
+        # Force b= to fold onto it's own line so that refolding after
+        # adding sig doesn't change whitespace for previous tags.
+        (b'b', b'0'*60), 
     ] if x]
     include_headers = [x.lower() for x in include_headers]
     # record what verify should extract
     self.include_headers = tuple(include_headers)
 
     sig_value = fold(b"; ".join(b"=".join(x) for x in sigfields))
+    sig_value = RE_BTAG.sub(b'\\1',sig_value)
     dkim_header = (b'DKIM-Signature', b' ' + sig_value)
-    h = hashlib.sha256()
+    h = hasher()
     sig = dict(sigfields)
     self.signed_headers = hash_headers(
-        h, canon_policy, headers, include_headers, [dkim_header],sig)
+        h, canon_policy, headers, include_headers, dkim_header,sig)
     self.logger.debug("sign headers: %r" % self.signed_headers)
 
     try:
         sig2 = RSASSA_PKCS1_v1_5_sign(h, pk)
     except DigestTooLargeError:
         raise ParameterError("digest too large for modulus")
-    sig_value += base64.b64encode(bytes(sig2))
+    # Folding b= is explicity allowed, but yahoo and live.com are broken
+    #sig_value += base64.b64encode(bytes(sig2))
+    # Instead of leaving unfolded (which lets an MTA fold it later and still
+    # breaks yahoo and live.com), we change the default signing mode to
+    # relaxed/simple (for broken receivers), and fold now.
+    sig_value = fold(sig_value + base64.b64encode(bytes(sig2)))
 
     self.domain = domain
     self.selector = selector
@@ -477,10 +485,10 @@ class DKIM(object):
     # By default, we validate the first DKIM-Signature line found.
     try:
         sig = parse_tag_value(sigheaders[idx][1])
+        self.signature_fields = sig
     except InvalidTagValueList as e:
         raise MessageFormatError(e)
 
-    sig = parse_tag_value(sigheaders[idx][1])
     logger = self.logger
     logger.debug("sig: %r" % sig)
 
@@ -540,16 +548,15 @@ class DKIM(object):
       include_headers.append('from')      
     h = hasher()
     self.signed_headers = hash_headers(
-        h, canon_policy, headers, include_headers, sigheaders, sig)
+        h, canon_policy, headers, include_headers, sigheaders[idx], sig)
     try:
-        self.signature_fields = sig
         signature = base64.b64decode(re.sub(br"\s+", b"", sig[b'b']))
         return RSASSA_PKCS1_v1_5_verify(h, signature, pk)
     except (TypeError,DigestTooLargeError) as e:
         raise KeyFormatError("digest too large for modulus: %s"%e)
 
 def sign(message, selector, domain, privkey, identity=None,
-         canonicalize=(b'simple', b'simple'),
+         canonicalize=(b'relaxed', b'simple'),
          signature_algorithm=b'rsa-sha256',
          include_headers=None, length=False, logger=None):
     """Sign an RFC822 message and return the DKIM-Signature header line.
@@ -582,5 +589,5 @@ def verify(message, logger=None, dnsfunc=get_txt):
         return d.verify(dnsfunc=dnsfunc)
     except DKIMException as x:
         if logger is not None:
-            logger.error("%s" % e)
+            logger.error("%s" % x)
         return False
